@@ -1,10 +1,13 @@
-package PDF::Table;
+#!/usr/bin/env perl
 
 use 5.006;
 use strict;
 use warnings;
+
+package PDF::Table;
+
 use Carp;
-our $VERSION = '0.9.13';
+our $VERSION = '0.10.0';
 
 print __PACKAGE__.' is version: '.$VERSION.$/ if($ENV{'PDF_TABLE_DEBUG'});
 
@@ -48,9 +51,9 @@ sub _init
 }
 
 sub set_defaults{
-	my $self = shift;
+    my $self = shift;
 
-	$self->{'font_size'} = 12;
+    $self->{'font_size'} = 12;
 }
 
 sub set_pdf{
@@ -287,7 +290,7 @@ sub table
 
     # Validate settings key
     my %valid_settings_key = (
-	x                     => 1,
+        x                     => 1,
         w                     => 1,
         start_y               => 1,
         start_h               => 1,
@@ -308,6 +311,7 @@ sub table
         vertical_borders      => 1,
         font                  => 1,
         font_size             => 1,
+        font_underline        => 1,
         font_color            => 1,
         font_color_even       => 1,
         font_color_odd        => 1,
@@ -320,6 +324,7 @@ sub table
         cell_props            => 1,
         max_word_length       => 1,
         cell_render_hook      => 1,
+        default_text          => 1,
     );
     foreach my $key (keys %arg)
     {
@@ -355,9 +360,10 @@ sub table
     my $txt     = $page->text;
 
     # Set Default Properties
-    my $fnt_name    = $arg{'font'            } || $pdf->corefont('Times',-encode => 'utf8');
-    my $fnt_size    = $arg{'font_size'       } || 12;
-    my $max_word_len= $arg{'max_word_length' } || 20;
+    my $fnt_name       = $arg{'font'            } || $pdf->corefont('Times',-encode => 'utf8');
+    my $fnt_size       = $arg{'font_size'       } || 12;
+    my $fnt_underline  = $arg{'font_underline'  } || undef; # merely stating undef is the intended default
+    my $max_word_len   = $arg{'max_word_length' } || 20;
 
     #=====================================
     # Table Header Section
@@ -376,6 +382,7 @@ sub table
         $header_props->{'font'          } = $header_props->{'font'          } || $fnt_name;
         $header_props->{'font_color'    } = $header_props->{'font_color'    } || '#000066';
         $header_props->{'font_size'     } = $header_props->{'font_size'     } || $fnt_size + 2;
+        $header_props->{'font_underline'} = $header_props->{'font_underline'} || $fnt_underline;
         $header_props->{'bg_color'      } = $header_props->{'bg_color'      } || '#FFFFAA';
         $header_props->{'justify'       } = $header_props->{'justify'       };
     }
@@ -389,6 +396,7 @@ sub table
     my $pad_right     = $arg{'padding_right' } || $arg{'padding'} || 0;
     my $pad_top       = $arg{'padding_top'   } || $arg{'padding'} || 0;
     my $pad_bot       = $arg{'padding_bottom'} || $arg{'padding'} || 0;
+    my $default_text  = $arg{'default_text'  } // '-';
     my $line_w        = defined $arg{'border'} ? $arg{'border'} : 1 ;
     my $horiz_borders = defined $arg{'horizontal_borders'}
         ? $arg{'horizontal_borders'}
@@ -437,7 +445,7 @@ sub table
 
     # Scalars that hold sum of the maximum and minimum widths of all columns
     my ( $max_col_w  , $min_col_w   ) = ( 0,0 );
-    my ( $row, $col_name, $col_fnt_size, $space_w );
+    my ( $row, $col_name, $col_fnt_size, $col_fnt_underline, $space_w );
 
     my $word_widths  = {};
     my $rows_height  = [];
@@ -452,12 +460,13 @@ sub table
         for( my $column_idx = 0; $column_idx < scalar(@{$data->[$row_idx]}) ; $column_idx++ )
         {
             # look for font information for this column
-            my ($cell_font, $cell_font_size);
+            my ($cell_font, $cell_font_size, $cell_font_underline);
 
             if( !$row_idx and ref $header_props )
             {
-                $cell_font      = $header_props->{'font'};
-                $cell_font_size = $header_props->{'font_size'};
+                $cell_font           = $header_props->{'font'};
+                $cell_font_size      = $header_props->{'font_size'};
+                $cell_font_underline = $header_props->{'font_underline'};
             }
 
             # Get the most specific value if none was already set from header_props
@@ -469,6 +478,12 @@ sub table
                             ||  $col_props->[$column_idx]->{'font_size'}
                             ||  $fnt_size;
 
+            $cell_font_underline ||= $cell_props->[$row_idx][$column_idx]->{'font_underline'}
+                                 ||  $col_props->[$column_idx]->{'font_underline'}
+                                 ||  $fnt_underline;
+
+            # Set Font
+
             # Set Font
             $txt->font( $cell_font, $cell_font_size );
 
@@ -543,7 +558,8 @@ sub table
     # Each iteration adds a new page as neccessary
     while(scalar(@{$data}))
     {
-        my ($page_header, $columns_number);
+        my ($page_header);
+        my $columns_number = 0;
 
         if($pg_cnt == 1)
         {
@@ -619,9 +635,9 @@ sub table
             # Remove the next item from $data
             my $record = shift @{$data};
 
-            # Get columns number to know later how many vertical lines to draw
-            # TODO: get the max number of columns per page as currently last row's columns overrides
-            $columns_number = scalar(@$record);
+            # Get max columns number to know later how many vertical lines to draw
+            $columns_number = scalar(@$record)
+                if scalar(@$record) > $columns_number;
 
             # Get the next set of row related settings
             # Row Height
@@ -634,11 +650,6 @@ sub table
 
             # Row cell props - TODO in another commit
 
-            # Added to resolve infite loop bug with returned undef values
-            for(my $d = 0; $d < scalar(@{$record}) ; $d++)
-            {
-                $record->[$d] = '-' unless( defined $record->[$d]);
-            }
 
             # Choose colors for this row
             $background_color = $row_index % 2 ? $background_color_even  : $background_color_odd;
@@ -669,14 +680,15 @@ sub table
                 $leftovers->[$column_idx] = undef;
 
                 # look for font information for this cell
-                my ($cell_font, $cell_font_size, $cell_font_color, $justify);
+                my ($cell_font, $cell_font_size, $cell_font_color, $cell_font_underline, $justify);
 
                 if( $first_row and ref $header_props)
                 {
-                    $cell_font       = $header_props->{'font'};
-                    $cell_font_size  = $header_props->{'font_size'};
-                    $cell_font_color = $header_props->{'font_color'};
-                    $justify         = $header_props->{'justify'};
+                    $cell_font           = $header_props->{'font'};
+                    $cell_font_size      = $header_props->{'font_size'};
+                    $cell_font_color     = $header_props->{'font_color'};
+                    $cell_font_underline = $header_props->{'font_underline'};
+                    $justify             = $header_props->{'justify'};
                 }
 
                 # Get the most specific value if none was already set from header_props
@@ -692,6 +704,11 @@ sub table
                                  ||  $col_props->[$column_idx]->{'font_color'}
                                  ||  $font_color;
 
+                $cell_font_underline ||= $cell_props->[$row_index][$column_idx]->{'font_underline'}
+                                     ||  $col_props->[$column_idx]->{'font_underline'}
+                                     ||  $fnt_underline;
+
+
                 $justify         ||= $cell_props->[$row_index][$column_idx]->{'justify'}
                                  ||  $col_props->[$column_idx]->{'justify'}
                                  ||  $arg{'justify'}
@@ -701,6 +718,11 @@ sub table
                 $txt->font( $cell_font, $cell_font_size );
                 $txt->fillcolor($cell_font_color);
 
+                # Added to resolve infite loop bug with returned undef values
+                $record->[$column_idx] //= $cell_props->[$row_index][$column_idx]->{'default_text'}
+                                       //  $col_props->[$column_idx]->{'default_text'}
+                                       //  $default_text;
+
                 # If the content is wider than the specified width, we need to add the text as a text block
                 if( $record->[$column_idx] !~ m/(.\n.)/ and
                     $record_widths->[$column_idx] and
@@ -716,7 +738,9 @@ sub table
                         $space = ($calc_column_widths->[$column_idx] - $txt->advancewidth($record->[$column_idx])) / 2;
                     }
                     $txt->translate( $cur_x + $space, $text_start );
-                    $txt->text( $record->[$column_idx] );
+                    my %text_options;
+                    $text_options{'-underline'} = $cell_font_underline if $cell_font_underline;
+                    $txt->text( $record->[$column_idx], %text_options );
                 }
                 # Otherwise just use the $page->text() method
                 else
@@ -1165,6 +1189,11 @@ B<Default:> 12
 
 =item B<font_color_even> - Font color for even rows
 
+=item B<font_underline> - Font underline of the header row
+
+B<Value:> 'auto', integer of distance, or arrayref of distance & thickness (more than one pair will provide mlultiple underlines. Negative distance gives strike-through.
+B<Default:> none
+
 =item B<background_color_odd> - Background color for odd rows
 
 =item B<background_color_even> - Background color for even rows
@@ -1247,6 +1276,11 @@ B<Default:> 'font_size' of the table + 2
 B<Value:> Color specifier as 'name' or 'HEX'
 B<Default:> '#000066'
 
+=item B<font_underline> - Font underline of the header row
+
+B<Value:> 'auto', integer of distance, or arrayref of distance & thickness (more than one pair will provide mlultiple underlines. Negative distance gives strike-through.
+B<Default:> none
+
 =item B<bg_color> - Background color of the header row
 
 B<Value:> Color specifier as 'name' or 'HEX'
@@ -1309,6 +1343,11 @@ B<Default:> 'font_size' of the table.
 B<Value:> Color specifier as 'name' or 'HEX'
 B<Default:> 'font_color' of the table.
 
+=item B<font_underline> - Font underline of this cell
+
+B<Value:> 'auto', integer of distance, or arrayref of distance & thickness (more than one pair will provide mlultiple underlines. Negative distance gives strike-through.
+B<Default:> none
+
 =item B<background_color> - Background color of this column
 
 B<Value:> Color specifier as 'name' or 'HEX'
@@ -1365,6 +1404,11 @@ B<Default:> 'font_size' of the table.
 B<Value:> Color specifier as 'name' or 'HEX'
 B<Default:> 'font_color' of the table.
 
+=item B<font_underline> - Font underline of this cell
+
+B<Value:> 'auto', integer of distance, or arrayref of distance & thickness (more than one pair will provide mlultiple underlines. Negative distance gives strike-through.
+B<Default:> none
+
 =item B<background_color> - Background color of this cell
 
 B<Value:> Color specifier as 'name' or 'HEX'
@@ -1382,6 +1426,7 @@ Example:
             {    #Row 1 cell 1
                 background_color => '#AAAA00',
                 font_color       => 'yellow',
+                font_underline   => [ 2, 2 ],
             },
 
             # etc.
